/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import java.util.Properties;

import org.apache.log4j.Logger;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

import edu.columbia.cs.dyswis.common.baseclass.DysProbe;
import edu.columbia.cs.dyswis.service.DyswisService;
import edu.columbia.cs.dyswis.service.MainService;

/**
 * Bundle activator for the module_mcast_rtp package.
 * 
 * @author phil
 */
public class Activator implements BundleActivator {

    /** logger */
    private static Logger logger = Logger.getLogger("Module McastRtp");

    /** The name of this package */
    private static final String packageName = "module_mcast_rtp";

    /**
     * The execution context of this bundle
     */
    private BundleContext m_context;

    /**
     * Dyswis Core Service tracker
     */
    public static ServiceTracker m_tracker;

    /**
     * Dyswis main service tracker
     */
    public static ServiceTracker m_mainTracker;

    /**
     * Multicast RTP detection module
     */
    public static McastRtpDetect mcastRtpDetect = null;

    /**
     * Multicast RTP fault listener
     */
    public static McastRtpFaultListener mcastRtpFaultListener = null;

    /**
     * Get a handle on the DyswisService
     * 
     * @return the DyswisService
     */
    public static DyswisService getDyswis() {
        return (DyswisService) m_tracker.getService();
    }

    /**
     * Get a handle on the MainService
     * 
     * @return the MainService
     */
    public static MainService getMain() {
        return (MainService) m_mainTracker.getService();
    }

    /**
     * Start the module_mcast_rtp bundle
     * <p>
     * Registers probes, starts the fault listener, and starts the detection
     * module.
     */
    @Override
    public void start(BundleContext arg0) throws Exception {

        logger.debug("Module McastRtp loading..");

        m_context = arg0;
        m_tracker =
                new ServiceTracker(m_context,
                        m_context.createFilter("(&(objectClass="
                                + DyswisService.class.getName() + ")"
                                + "(Name=DyswisService))"), null);
        m_tracker.open();

        m_mainTracker =
                new ServiceTracker(m_context,
                        m_context.createFilter("(&(objectClass="
                                + MainService.class.getName() + ")"
                                + "(Name=MainService))"), null);
        m_mainTracker.open();

        logger.debug("Module McastRtp has been loaded");

        /*
         * register the probes with the DyswisService
         */
        contextRegisterService("McastRtpProbe", m_context);
        contextRegisterService("McastRtpFaultHistoryProbe", m_context);
        contextRegisterService("McastRtpReversePathDiscovery",
                m_context);
        contextRegisterService("McastRtpSessionHistoryProbe", m_context);
        contextRegisterService("McastRtpRemoteProbe", m_context);

        /*
         * Start the McastRtpFaultListener
         */
        mcastRtpFaultListener = new McastRtpFaultListener();

        /*
         * Start the McastRtpDetect module
         */
        mcastRtpDetect = new McastRtpDetect();
        getMain().addPacketCapture(mcastRtpDetect);
    }

    /**
     * Stops the module_mcast_rtp bundle
     */
    @Override
    public void stop(BundleContext arg0) throws Exception {

        logger.debug("Module McastRtp is being unloading..");
        getMain().removePacketCapture(mcastRtpDetect);
        mcastRtpFaultListener = null;
        mcastRtpDetect = null;

        m_tracker.close();
        m_mainTracker.close();
        logger.debug("Module McastRtp has been unloaded");
    }

    /**
     * Register the probe.
     * 
     * @param className
     *            java class of the probe
     * @param context
     *            the execution context of the bundle
     */
    public void contextRegisterService(String className,
            BundleContext context) {

        Properties props = new Properties();

        props.put("Package", packageName);
        props.put("Probe", className);

        String classFullPath =
                "edu.columbia.cs.dyswis." + packageName + "."
                        + className;
        try {

            Class<?> obj;
            obj = Class.forName(classFullPath);
            DysProbe dp = (DysProbe) obj.newInstance();
            context.registerService(DysProbe.class.getName(), dp, props);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}