/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import java.util.HashMap;

import org.apache.log4j.Logger;

import edu.columbia.cs.dyswis.common.baseclass.DysProbe;
import edu.columbia.cs.dyswis.service.DyswisService;

/**
 * Query the Fault History to see if a loss event was detected.
 * <p>
 * At one point, this was intended to be called directly as a remote probe, and
 * it can still be used in this way. However, now it is called locally (by the
 * remote receiver) as part of the McastRtpRemoteProbe.
 * 
 * @author phil
 */
public class McastRtpFaultHistoryProbe implements DysProbe {

    /** for log messages */
    private static Logger logger = Logger
            .getLogger("McastRtpFaultHistoryProbe");

    /**
     * Check the history for a given McastRtpFault
     * 
     * @param faultMap
     *            either an McastRtpFault, or a HashMap containing an entry with
     *            a key of "FAULT" that maps to a fault string
     * @return true if the fault was found, or false otherwise
     */
    @Override
    public Object probe(Object faultMap) {

        logger.debug("probe: " + faultMap);

        String faultString;

        if (faultMap instanceof HashMap<?, ?>) {
            logger.debug("hashmap detected");
            faultString =
                    (String) ((HashMap<?, ?>) faultMap).get("FAULT");
        } else if (faultMap instanceof McastRtpFault) {
            faultString = faultMap.toString();
        } else {
            logger.debug("Unknown fault type: " + faultMap.toString());
            return null;
        }

        logger.debug("checking fault history for: " + faultString);
        return Activator.mcastRtpFaultListener.findFault(faultString);
    }

    /**
     * Add a message to the GUI with the result of the probe
     * 
     * @param request
     *            the initial request that was sent to the remote
     * @param result
     *            the result of the probe
     */
    @Override
    public void displayResult(Object request, Object result) {

        DyswisService gui = Activator.getDyswis();
        gui.addDiagnosisMessageTitle("Multicast RTP Fault History Probe");
        if ((Boolean) result) {
            gui.addDiagnosisMessageContent("Remote side found fault", 0);
        } else {
            gui.addDiagnosisMessageContent(
                    "Remote side didn't find fault", 0);
        }
    }

}
