/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.Logger;

import edu.columbia.cs.dyswis.common.baseclass.DysFault;

/**
 * Maintains a searchable history of McastRtpFaults.
 * <p>
 * I'd really like this to implement the
 * FaultListener interface,
 * so that faults that are added via addFault are
 * automatically added here too.
 * But, that interface isn't exported, and
 * I don't know if all the hooks would work.
 * 
 * @see edu.columbia.cs.dyswis.core.diagnosis.FaultListener
 * @author phil
 */
public class McastRtpFaultListener /* implements FaultListener */ {

	private static Logger logger = Logger.getLogger("McastRtpFaultListener");

	private Map<String,String> faultMap;
	
	/**
	 * Default constructor
	 */
	public McastRtpFaultListener() {
		
		logger.debug("McastRtpFaultListener started");
		faultMap = new ConcurrentHashMap<String,String>();
	}

	/**
	 * Right now this is called by faultAdded in McastRtpSession
	 * @see edu.columbia.cs.dyswis.module_mcast_rtp.McastRtpSession#addFault
	 */
	public void faultAdded(DysFault fault) {
		
		if (!(fault instanceof McastRtpFault)) {
			return;
		}
		
		logger.debug("fault added: " + fault.toString() + ": ts :" + fault.getTimestamp());
		faultMap.put(fault.toString(), fault.getTimestamp());
	}

	
	public boolean findFault(String faultString) {

		logger.debug("finding fault: " + faultString);

		if (faultMap.containsKey(faultString)) {
			logger.debug("found!");
			return true;
		}
		else {
			logger.debug("not found!");
			return false;
		}
}
	
	public boolean findFault(McastRtpFault fault) {
		if (fault == null) {
			logger.debug("fault is null");
			return false;
		}
		String faultString = fault.toString();
		return findFault(faultString);
	}
	

}
