/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import java.util.HashMap;

import org.apache.log4j.Logger;

import edu.columbia.cs.dyswis.common.baseclass.DysProbe;
import edu.columbia.cs.dyswis.service.DyswisService;

/**
 * Local probe that simply displays information about a McastRtpFault.
 * <p>
 * Specifically, it displays the Source IP and port, the Destination multicast
 * group IP and port, the expected RTP sequence number, and the received RTP
 * sequence number.
 * 
 * @author phil
 */
public class McastRtpProbe implements DysProbe {

    /** for log messages */
    private static Logger logger = Logger.getLogger("McastRtpProbe");

    /**
     * Display information about an McastRtpFault
     * @param faultMap the McastRtpFault
     * @return true if the input was a valid McastRtpFault, false otherwise
     */
    @Override
    public Object probe(Object faultMap) {

        logger.debug("probe: " + faultMap);

        DyswisService gui = Activator.getDyswis();
        gui.resetDiagnoseMessage();
        gui.addDiagnosisMessageTitle("Multicast RTP");

        McastRtpFault fault;

        if (faultMap instanceof HashMap<?, ?>) {
            /* don't do anything for a remote probe */
            return false;
        } else if (faultMap instanceof McastRtpFault) {
            /* normal case - a local probe */
            fault = (McastRtpFault) faultMap;
        } else {
            /* not sure why we would get this */
            gui.addDiagnosisMessageContent("Unknown fault type", 0);
            return false;
        }

        /*
         * Display the information in the GUI
         */
        gui.addDiagnosisMessageContent("RTP Sequence Number Error", 0);
        gui.addDiagnosisMessageDescribe(
                "Source: " + fault.getHostAddr() + "/"
                        + fault.getHostPort(), 0);
        gui.addDiagnosisMessageDescribe(
                "Destination: " + fault.getMyAddr() + "/"
                        + fault.getMyPort(), 0);
        gui.addDiagnosisMessageDescribe(
                "Expected RTP Sequence Number: " + fault.getExpected(),
                0);
        gui.addDiagnosisMessageDescribe(
                "Received RTP Sequence Number: " + fault.getReceived(),
                1);

        return true;
    }

    /**
     * Required override. Not used.
     */
    @Override
    public void displayResult(Object request, Object result) {

        logger.debug("display result: " + request + ": " + result);

    }

}
