/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import org.apache.log4j.Logger;

import edu.columbia.cs.dyswis.common.baseclass.DysProbe;
import edu.columbia.cs.dyswis.service.DyswisService;

/**
 * Monolithic remote probe that consolidates all the data that may relate to a
 * fault.
 * <p>
 * Since there is no concept of a "conversation" between nodes, we cannot
 * guarantee that multiple remote probes executed in series will talk to the
 * same remote. Therefore, it is best to get all the data via a single remote
 * query.
 * <p>
 * This probe is intended to be called remotely, either by RemoteProbe, or most
 * likely as by MultiProbe.
 * 
 * @author phil
 */
public class McastRtpRemoteProbe implements DysProbe {

    /** for logging */
    private static Logger logger = Logger
            .getLogger("McastRtpRemoteProbe");

    /**
     * Given a hash map relating to an McastRtpFault, returns a string
     * describing what the remote probe experienced.
     * <p>
     * This is a little kludgy right now, but the String contains the following:
     * <br>
     * <SAME_SESSION> <SAME_FAULT> <PATH_TO_SOURCE><br>
     * where<br>
     * SAME_SESSION => "true" or "false"<br>
     * SAME_FAULT => "true" or "false"<br>
     * PATH_TO_SOURCE => space separated traceroute values<br>
     * 
     * @return a String describing what the remote probe experienced.
     */
    @Override
    public Object probe(Object faultMap) {
        logger.debug("probe called!");
        logger.debug(faultMap.toString());

        DysProbe probe;
        probe = new McastRtpSessionHistoryProbe();
        boolean sameSession = (Boolean) probe.probe(faultMap);

        probe = new McastRtpFaultHistoryProbe();
        boolean sameFault = (Boolean) probe.probe(faultMap);

        probe = new McastRtpReversePathDiscovery();
        String result = (String) probe.probe(faultMap);

        return new String(sameSession + " " + sameFault + " " + result);
    }

    /**
     * Called by MultiProbe (or RemoteProbe, theoretically) to display the
     * result.
     */
    @Override
    public void displayResult(Object request, Object result) {

        if (result == null || !(result instanceof String)) {
            return;
        }
        
        /* parse the result */
        String[] fields = ((String) result).split(" ", 3);
        if (fields.length < 3)
            return;

        /* display the GUI messages */
        DyswisService gui = Activator.getDyswis();
        if (gui != null) {
            gui.addDiagnosisMessageDescribe("Same session: "
                    + fields[0], 0);
            gui.addDiagnosisMessageDescribe("Same fault: " + fields[1],
                    0);
            gui.addDiagnosisMessageDescribe("Hops to source: "
                    + fields[2], 0);
        }
    }

}
